'' =================================================================================================
''
''   File....... jm_sircs_rx.spin
''   Purpose.... SIRCS receiver
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 27 APR 2009
''   Updated.... 21 JUL 2009
''
'' =================================================================================================

{{

    IR Connection (PNA4602M)
 
                ┌───┐
                │(*)│ 5v
                └┬┬┬┘ 
    irPin ────┘│└──┘
             2k2  

}}         


var

  long  cog                                                     ' cog id

  long  irReady                                                 ' has ir code
  long  irCode                                                  ' last code rx'd
  long  irBits                                                  ' bits in code


pub init(p) | okay

'' Setup IR input on pin p

  finalize

  if (p => 0) and (p =< 29)                                     ' protect rx, tx, i2c
    START_BIT := clkfreq / 1_000_000 * 1_920                    ' set sircs rx cog parameters
    ONE_BIT   := clkfreq / 1_000_000 *   960
    MS_044    := clkfreq / 1_000 * 44       
    irmask    := |< p
    okpntr    := @irReady   
    codepntr  := @irCode
    bitspntr  := @irBits

    irReady := true                                             ' disable driver
    irCode := 0
    irBits := 0

    okay := cog := cognew(@rxsircs, 0) + 1                      ' start cog

  return okay
  

pub finalize

  if cog
    cogstop(cog~ - 1)


pub getir

'' Enables and waits for ir input
'' -- warning: blocks until IR code received!

  enable                                                        ' start ir rx
  repeat while !irReady                                         ' let it finish
  return irCode                                                 ' return code


pub enable

'' Enables IR receive process

  irBits := 0
  irCode := 0
  irReady := false


pub status

'' Reports status of IR receiver

  return (irReady > 0)


pub rx

'' Returns last rx'd ir code
'' -- check status before using

  return irCode


pub bitcount

'' Returns bit count of last ir code
'' -- check status before using

  return irBits
      

dat

                        org     0

rxsircs                 mov     ctra, NEG_DETECT                ' set ctra for pulse timing
                        mov     frqa, #1

                        mov     ctrb, FREE_RUN                  ' set ctrb for frame timing
                        mov     frqb, #1                 

waitok                  rdlong  tmp1, okpntr            wz      ' wait for zero
                if_nz   jmp     #waitok

waitstart               waitpeq irmask, irmask                  ' wait for high
                        mov     phsa, #0                        ' start bit timer
                        waitpne irmask, irmask                  ' wait for low
                        mov     phsb, #0                        ' start frame timer
                        waitpeq irmask, irmask                  ' wait for high
                        cmp     START_BIT, phsa         wc      ' valid start bit?
                if_nc   jmp     #waitstart                      ' try again if no

                        mov     irwork, #0
                        mov     bits, #0                        ' reset bit count

checkframe              cmp     MS_044, phsb            wc      ' check frame timer
                if_c    jmp     #irdone                         ' abort @44ms

waitbit                 test    irmask, ina             wz      ' look for new bit
                if_nz   jmp     #checkframe                     ' check for end if no bit
        
measurebit              mov     phsa, #0                        ' resstart bit timer
                        waitpeq irmask, irmask                  ' wait for high (end of bit)  
                        cmp     ONE_BIT, phsa           wc      ' ir bit --> C
                        rcr     irwork, #1                      ' C --> irwork.31
                
                        add     bits, #1                        ' inc bit count
                        cmp     bits, #20               wc      ' at max?
                if_b    jmp     #checkframe                     ' keep scanning if no
                        
irdone                  mov     tmp1, #32
                        sub     tmp1, bits
                        shr     irwork, tmp1                    ' right align ir code  

report                  wrlong  irwork, codepntr                ' write code to hub
                        wrlong  bits, bitspntr                  ' write bit count to hub
                        wrlong  DONE, okpntr                    ' alert caller
                        
                        jmp     #waitok
                        
' -------------------------------------------------------------------------------------------------

DONE                    long    true 

NEG_DETECT              long    %01100 << 26                    ' ctr neg detector
FREE_RUN                long    %11111 << 26                    ' ctr logic always mode

START_BIT               long    0-0                             ' counts in 80% of 2.4ms
ONE_BIT                 long    0-0                             ' counts in 80% of 1.2ms
MS_044                  long    0-0                             ' counts in 44ms   

irmask                  long    0-0                             ' pin mask for ir input
okpntr                  long    0-0                             ' pointer to ready flag
codepntr                long    0-0                             ' pointer to ir code
bitspntr                long    0-0                             ' pointer to bit count of code

irwork                  res     1                               ' workspace for ir input
bits                    res     1                               ' # bits in input

tmp1                    res     1
tmp2                    res     1

                        fit     492


dat

{{

  Copyright (c) 2009 Jon McPhalen (aka Jon Williams)

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}                     
